@@ -30,6 +30,8 @@
 #include <libs/mathlib.h>
 #include <cpu_defines.h>
 #include <cpu_inline_asm.h>
+#include <cpu_vcpu_vfp.h>
+#include <cpu_vcpu_cp14.h>
 #include <cpu_vcpu_cp15.h>
 #include <cpu_vcpu_helper.h>
 #include <arm_features.h>
@@ -61,285 +63,290 @@ u32 cpu_vcpu_cpsr_retrieve(struct vmm_vcpu *vcpu,
 	}
 }
 
-void cpu_vcpu_banked_regs_save(struct vmm_vcpu *vcpu, arch_regs_t *src)
+static void cpu_vcpu_banked_regs_save(struct arm_priv *p, arch_regs_t *src)
 {
-	if (!vcpu || !vcpu->is_normal || !src) {
-		return;
-	}
-	switch (arm_priv(vcpu)->cpsr & CPSR_MODE_MASK) {
+	switch (p->cpsr & CPSR_MODE_MASK) {
 	case CPSR_MODE_USER:
-		arm_priv(vcpu)->gpr_usr[0] = src->gpr[8];
-		arm_priv(vcpu)->gpr_usr[1] = src->gpr[9];
-		arm_priv(vcpu)->gpr_usr[2] = src->gpr[10];
-		arm_priv(vcpu)->gpr_usr[3] = src->gpr[11];
-		arm_priv(vcpu)->gpr_usr[4] = src->gpr[12];
-		arm_priv(vcpu)->sp_usr = src->sp;
-		arm_priv(vcpu)->lr_usr = src->lr;
+		p->gpr_usr[0] = src->gpr[8];
+		p->gpr_usr[1] = src->gpr[9];
+		p->gpr_usr[2] = src->gpr[10];
+		p->gpr_usr[3] = src->gpr[11];
+		p->gpr_usr[4] = src->gpr[12];
+		p->sp_usr = src->sp;
+		p->lr_usr = src->lr;
 		break;
 	case CPSR_MODE_SYSTEM:
-		arm_priv(vcpu)->gpr_usr[0] = src->gpr[8];
-		arm_priv(vcpu)->gpr_usr[1] = src->gpr[9];
-		arm_priv(vcpu)->gpr_usr[2] = src->gpr[10];
-		arm_priv(vcpu)->gpr_usr[3] = src->gpr[11];
-		arm_priv(vcpu)->gpr_usr[4] = src->gpr[12];
-		arm_priv(vcpu)->sp_usr = src->sp;
-		arm_priv(vcpu)->lr_usr = src->lr;
+		p->gpr_usr[0] = src->gpr[8];
+		p->gpr_usr[1] = src->gpr[9];
+		p->gpr_usr[2] = src->gpr[10];
+		p->gpr_usr[3] = src->gpr[11];
+		p->gpr_usr[4] = src->gpr[12];
+		p->sp_usr = src->sp;
+		p->lr_usr = src->lr;
 		break;
 	case CPSR_MODE_ABORT:
-		arm_priv(vcpu)->gpr_usr[0] = src->gpr[8];
-		arm_priv(vcpu)->gpr_usr[1] = src->gpr[9];
-		arm_priv(vcpu)->gpr_usr[2] = src->gpr[10];
-		arm_priv(vcpu)->gpr_usr[3] = src->gpr[11];
-		arm_priv(vcpu)->gpr_usr[4] = src->gpr[12];
-		arm_priv(vcpu)->sp_abt = src->sp;
-		arm_priv(vcpu)->lr_abt = src->lr;
+		p->gpr_usr[0] = src->gpr[8];
+		p->gpr_usr[1] = src->gpr[9];
+		p->gpr_usr[2] = src->gpr[10];
+		p->gpr_usr[3] = src->gpr[11];
+		p->gpr_usr[4] = src->gpr[12];
+		p->sp_abt = src->sp;
+		p->lr_abt = src->lr;
 		break;
 	case CPSR_MODE_UNDEFINED:
-		arm_priv(vcpu)->gpr_usr[0] = src->gpr[8];
-		arm_priv(vcpu)->gpr_usr[1] = src->gpr[9];
-		arm_priv(vcpu)->gpr_usr[2] = src->gpr[10];
-		arm_priv(vcpu)->gpr_usr[3] = src->gpr[11];
-		arm_priv(vcpu)->gpr_usr[4] = src->gpr[12];
-		arm_priv(vcpu)->sp_und = src->sp;
-		arm_priv(vcpu)->lr_und = src->lr;
+		p->gpr_usr[0] = src->gpr[8];
+		p->gpr_usr[1] = src->gpr[9];
+		p->gpr_usr[2] = src->gpr[10];
+		p->gpr_usr[3] = src->gpr[11];
+		p->gpr_usr[4] = src->gpr[12];
+		p->sp_und = src->sp;
+		p->lr_und = src->lr;
 		break;
 	case CPSR_MODE_MONITOR:
-		arm_priv(vcpu)->gpr_usr[0] = src->gpr[8];
-		arm_priv(vcpu)->gpr_usr[1] = src->gpr[9];
-		arm_priv(vcpu)->gpr_usr[2] = src->gpr[10];
-		arm_priv(vcpu)->gpr_usr[3] = src->gpr[11];
-		arm_priv(vcpu)->gpr_usr[4] = src->gpr[12];
-		arm_priv(vcpu)->sp_mon = src->sp;
-		arm_priv(vcpu)->lr_mon = src->lr;
+		p->gpr_usr[0] = src->gpr[8];
+		p->gpr_usr[1] = src->gpr[9];
+		p->gpr_usr[2] = src->gpr[10];
+		p->gpr_usr[3] = src->gpr[11];
+		p->gpr_usr[4] = src->gpr[12];
+		p->sp_mon = src->sp;
+		p->lr_mon = src->lr;
 		break;
 	case CPSR_MODE_SUPERVISOR:
-		arm_priv(vcpu)->gpr_usr[0] = src->gpr[8];
-		arm_priv(vcpu)->gpr_usr[1] = src->gpr[9];
-		arm_priv(vcpu)->gpr_usr[2] = src->gpr[10];
-		arm_priv(vcpu)->gpr_usr[3] = src->gpr[11];
-		arm_priv(vcpu)->gpr_usr[4] = src->gpr[12];
-		arm_priv(vcpu)->sp_svc = src->sp;
-		arm_priv(vcpu)->lr_svc = src->lr;
+		p->gpr_usr[0] = src->gpr[8];
+		p->gpr_usr[1] = src->gpr[9];
+		p->gpr_usr[2] = src->gpr[10];
+		p->gpr_usr[3] = src->gpr[11];
+		p->gpr_usr[4] = src->gpr[12];
+		p->sp_svc = src->sp;
+		p->lr_svc = src->lr;
 		break;
 	case CPSR_MODE_IRQ:
-		arm_priv(vcpu)->gpr_usr[0] = src->gpr[8];
-		arm_priv(vcpu)->gpr_usr[1] = src->gpr[9];
-		arm_priv(vcpu)->gpr_usr[2] = src->gpr[10];
-		arm_priv(vcpu)->gpr_usr[3] = src->gpr[11];
-		arm_priv(vcpu)->gpr_usr[4] = src->gpr[12];
-		arm_priv(vcpu)->sp_irq = src->sp;
-		arm_priv(vcpu)->lr_irq = src->lr;
+		p->gpr_usr[0] = src->gpr[8];
+		p->gpr_usr[1] = src->gpr[9];
+		p->gpr_usr[2] = src->gpr[10];
+		p->gpr_usr[3] = src->gpr[11];
+		p->gpr_usr[4] = src->gpr[12];
+		p->sp_irq = src->sp;
+		p->lr_irq = src->lr;
 		break;
 	case CPSR_MODE_FIQ:
-		arm_priv(vcpu)->gpr_fiq[0] = src->gpr[8];
-		arm_priv(vcpu)->gpr_fiq[1] = src->gpr[9];
-		arm_priv(vcpu)->gpr_fiq[2] = src->gpr[10];
-		arm_priv(vcpu)->gpr_fiq[3] = src->gpr[11];
-		arm_priv(vcpu)->gpr_fiq[4] = src->gpr[12];
-		arm_priv(vcpu)->sp_fiq = src->sp;
-		arm_priv(vcpu)->lr_fiq = src->lr;
+		p->gpr_fiq[0] = src->gpr[8];
+		p->gpr_fiq[1] = src->gpr[9];
+		p->gpr_fiq[2] = src->gpr[10];
+		p->gpr_fiq[3] = src->gpr[11];
+		p->gpr_fiq[4] = src->gpr[12];
+		p->sp_fiq = src->sp;
+		p->lr_fiq = src->lr;
 		break;
 	default:
 		break;
 	};
 }
 
-void cpu_vcpu_banked_regs_restore(struct vmm_vcpu *vcpu, arch_regs_t *dst)
+static void cpu_vcpu_banked_regs_restore(struct arm_priv *p, arch_regs_t *dst)
 {
-	if (!vcpu || !vcpu->is_normal || !dst) {
-		return;
-	}
-	switch (arm_priv(vcpu)->cpsr & CPSR_MODE_MASK) {
+	switch (p->cpsr & CPSR_MODE_MASK) {
 	case CPSR_MODE_USER:
-		dst->gpr[8] = arm_priv(vcpu)->gpr_usr[0];
-		dst->gpr[9] = arm_priv(vcpu)->gpr_usr[1];
-		dst->gpr[10] = arm_priv(vcpu)->gpr_usr[2];
-		dst->gpr[11] = arm_priv(vcpu)->gpr_usr[3];
-		dst->gpr[12] = arm_priv(vcpu)->gpr_usr[4];
-		dst->sp = arm_priv(vcpu)->sp_usr;
-		dst->lr = arm_priv(vcpu)->lr_usr;
+		dst->gpr[8] = p->gpr_usr[0];
+		dst->gpr[9] = p->gpr_usr[1];
+		dst->gpr[10] = p->gpr_usr[2];
+		dst->gpr[11] = p->gpr_usr[3];
+		dst->gpr[12] = p->gpr_usr[4];
+		dst->sp = p->sp_usr;
+		dst->lr = p->lr_usr;
 		break;
 	case CPSR_MODE_SYSTEM:
-		dst->gpr[8] = arm_priv(vcpu)->gpr_usr[0];
-		dst->gpr[9] = arm_priv(vcpu)->gpr_usr[1];
-		dst->gpr[10] = arm_priv(vcpu)->gpr_usr[2];
-		dst->gpr[11] = arm_priv(vcpu)->gpr_usr[3];
-		dst->gpr[12] = arm_priv(vcpu)->gpr_usr[4];
-		dst->sp = arm_priv(vcpu)->sp_usr;
-		dst->lr = arm_priv(vcpu)->lr_usr;
+		dst->gpr[8] = p->gpr_usr[0];
+		dst->gpr[9] = p->gpr_usr[1];
+		dst->gpr[10] = p->gpr_usr[2];
+		dst->gpr[11] = p->gpr_usr[3];
+		dst->gpr[12] = p->gpr_usr[4];
+		dst->sp = p->sp_usr;
+		dst->lr = p->lr_usr;
 		break;
 	case CPSR_MODE_ABORT:
-		dst->gpr[8] = arm_priv(vcpu)->gpr_usr[0];
-		dst->gpr[9] = arm_priv(vcpu)->gpr_usr[1];
-		dst->gpr[10] = arm_priv(vcpu)->gpr_usr[2];
-		dst->gpr[11] = arm_priv(vcpu)->gpr_usr[3];
-		dst->gpr[12] = arm_priv(vcpu)->gpr_usr[4];
-		dst->sp = arm_priv(vcpu)->sp_abt;
-		dst->lr = arm_priv(vcpu)->lr_abt;
+		dst->gpr[8] = p->gpr_usr[0];
+		dst->gpr[9] = p->gpr_usr[1];
+		dst->gpr[10] = p->gpr_usr[2];
+		dst->gpr[11] = p->gpr_usr[3];
+		dst->gpr[12] = p->gpr_usr[4];
+		dst->sp = p->sp_abt;
+		dst->lr = p->lr_abt;
 		break;
 	case CPSR_MODE_UNDEFINED:
-		dst->gpr[8] = arm_priv(vcpu)->gpr_usr[0];
-		dst->gpr[9] = arm_priv(vcpu)->gpr_usr[1];
-		dst->gpr[10] = arm_priv(vcpu)->gpr_usr[2];
-		dst->gpr[11] = arm_priv(vcpu)->gpr_usr[3];
-		dst->gpr[12] = arm_priv(vcpu)->gpr_usr[4];
-		dst->sp = arm_priv(vcpu)->sp_und;
-		dst->lr = arm_priv(vcpu)->lr_und;
+		dst->gpr[8] = p->gpr_usr[0];
+		dst->gpr[9] = p->gpr_usr[1];
+		dst->gpr[10] = p->gpr_usr[2];
+		dst->gpr[11] = p->gpr_usr[3];
+		dst->gpr[12] = p->gpr_usr[4];
+		dst->sp = p->sp_und;
+		dst->lr = p->lr_und;
 		break;
 	case CPSR_MODE_MONITOR:
-		dst->gpr[8] = arm_priv(vcpu)->gpr_usr[0];
-		dst->gpr[9] = arm_priv(vcpu)->gpr_usr[1];
-		dst->gpr[10] = arm_priv(vcpu)->gpr_usr[2];
-		dst->gpr[11] = arm_priv(vcpu)->gpr_usr[3];
-		dst->gpr[12] = arm_priv(vcpu)->gpr_usr[4];
-		dst->sp = arm_priv(vcpu)->sp_mon;
-		dst->lr = arm_priv(vcpu)->lr_mon;
+		dst->gpr[8] = p->gpr_usr[0];
+		dst->gpr[9] = p->gpr_usr[1];
+		dst->gpr[10] = p->gpr_usr[2];
+		dst->gpr[11] = p->gpr_usr[3];
+		dst->gpr[12] = p->gpr_usr[4];
+		dst->sp = p->sp_mon;
+		dst->lr = p->lr_mon;
 		break;
 	case CPSR_MODE_SUPERVISOR:
-		dst->gpr[8] = arm_priv(vcpu)->gpr_usr[0];
-		dst->gpr[9] = arm_priv(vcpu)->gpr_usr[1];
-		dst->gpr[10] = arm_priv(vcpu)->gpr_usr[2];
-		dst->gpr[11] = arm_priv(vcpu)->gpr_usr[3];
-		dst->gpr[12] = arm_priv(vcpu)->gpr_usr[4];
-		dst->sp = arm_priv(vcpu)->sp_svc;
-		dst->lr = arm_priv(vcpu)->lr_svc;
+		dst->gpr[8] = p->gpr_usr[0];
+		dst->gpr[9] = p->gpr_usr[1];
+		dst->gpr[10] = p->gpr_usr[2];
+		dst->gpr[11] = p->gpr_usr[3];
+		dst->gpr[12] = p->gpr_usr[4];
+		dst->sp = p->sp_svc;
+		dst->lr = p->lr_svc;
 		break;
 	case CPSR_MODE_IRQ:
-		dst->gpr[8] = arm_priv(vcpu)->gpr_usr[0];
-		dst->gpr[9] = arm_priv(vcpu)->gpr_usr[1];
-		dst->gpr[10] = arm_priv(vcpu)->gpr_usr[2];
-		dst->gpr[11] = arm_priv(vcpu)->gpr_usr[3];
-		dst->gpr[12] = arm_priv(vcpu)->gpr_usr[4];
-		dst->sp = arm_priv(vcpu)->sp_irq;
-		dst->lr = arm_priv(vcpu)->lr_irq;
+		dst->gpr[8] = p->gpr_usr[0];
+		dst->gpr[9] = p->gpr_usr[1];
+		dst->gpr[10] = p->gpr_usr[2];
+		dst->gpr[11] = p->gpr_usr[3];
+		dst->gpr[12] = p->gpr_usr[4];
+		dst->sp = p->sp_irq;
+		dst->lr = p->lr_irq;
 		break;
 	case CPSR_MODE_FIQ:
-		dst->gpr[8] = arm_priv(vcpu)->gpr_fiq[0];
-		dst->gpr[9] = arm_priv(vcpu)->gpr_fiq[1];
-		dst->gpr[10] = arm_priv(vcpu)->gpr_fiq[2];
-		dst->gpr[11] = arm_priv(vcpu)->gpr_fiq[3];
-		dst->gpr[12] = arm_priv(vcpu)->gpr_fiq[4];
-		dst->sp = arm_priv(vcpu)->sp_fiq;
-		dst->lr = arm_priv(vcpu)->lr_fiq;
+		dst->gpr[8] = p->gpr_fiq[0];
+		dst->gpr[9] = p->gpr_fiq[1];
+		dst->gpr[10] = p->gpr_fiq[2];
+		dst->gpr[11] = p->gpr_fiq[3];
+		dst->gpr[12] = p->gpr_fiq[4];
+		dst->sp = p->sp_fiq;
+		dst->lr = p->lr_fiq;
 		break;
 	default:
 		break;
 	};
 }
 
-void cpu_vcpu_cpsr_update(struct vmm_vcpu *vcpu, 
+void cpu_vcpu_cpsr_update(struct vmm_vcpu *vcpu,
 			  arch_regs_t *regs,
 			  u32 new_cpsr,
 			  u32 new_cpsr_mask)
 {
 	bool mode_change;
+	struct arm_priv *p;
 
 	/* Sanity check */
 	if (!vcpu || !vcpu->is_normal || !regs) {
 		return;
 	}
+	p = arm_priv(vcpu);
 	new_cpsr &= new_cpsr_mask;
+
 	/* Determine if mode is changing */
 	mode_change = FALSE;
 	if ((new_cpsr_mask & CPSR_MODE_MASK) &&
-	    ((arm_priv(vcpu)->cpsr & CPSR_MODE_MASK) != 
-					(new_cpsr & CPSR_MODE_MASK))) {
+	    ((p->cpsr & CPSR_MODE_MASK) != (new_cpsr & CPSR_MODE_MASK))) {
 		mode_change = TRUE;
 		/* Save banked registers for old CPSR */
-		cpu_vcpu_banked_regs_save(vcpu, regs);
+		cpu_vcpu_banked_regs_save(p, regs);
 	}
+
 	/* Set the new priviledged bits of CPSR */
-	arm_priv(vcpu)->cpsr &= (~CPSR_PRIVBITS_MASK | ~new_cpsr_mask);
-	arm_priv(vcpu)->cpsr |= new_cpsr & CPSR_PRIVBITS_MASK & new_cpsr_mask;
+	p->cpsr &= (~CPSR_PRIVBITS_MASK | ~new_cpsr_mask);
+	p->cpsr |= new_cpsr & CPSR_PRIVBITS_MASK & new_cpsr_mask;
+
 	/* Set the new user bits of CPSR */
 	regs->cpsr &= (~CPSR_USERBITS_MASK | ~new_cpsr_mask);
 	regs->cpsr |= new_cpsr & CPSR_USERBITS_MASK & new_cpsr_mask;
+
 	/* If mode is changing then */
 	if (mode_change) {
 		/* Restore values of banked registers for new CPSR */
-		cpu_vcpu_banked_regs_restore(vcpu, regs);
+		cpu_vcpu_banked_regs_restore(p, regs);
 		/* Synchronize CP15 state to change in mode */
 		cpu_vcpu_cp15_sync_cpsr(vcpu);
 	}
+
 	return;
 }
 
 u32 cpu_vcpu_spsr_retrieve(struct vmm_vcpu *vcpu)
 {
+	struct arm_priv *p = arm_priv(vcpu);
+
 	/* Find out correct SPSR */
-	switch (arm_priv(vcpu)->cpsr & CPSR_MODE_MASK) {
+	switch (p->cpsr & CPSR_MODE_MASK) {
 	case CPSR_MODE_ABORT:
-		return arm_priv(vcpu)->spsr_abt;
-		break;
+		return p->spsr_abt;
 	case CPSR_MODE_UNDEFINED:
-		return arm_priv(vcpu)->spsr_und;
-		break;
+		return p->spsr_und;
 	case CPSR_MODE_MONITOR:
-		return arm_priv(vcpu)->spsr_mon;
-		break;
+		return p->spsr_mon;
 	case CPSR_MODE_SUPERVISOR:
-		return arm_priv(vcpu)->spsr_svc;
-		break;
+		return p->spsr_svc;
 	case CPSR_MODE_IRQ:
-		return arm_priv(vcpu)->spsr_irq;
-		break;
+		return p->spsr_irq;
 	case CPSR_MODE_FIQ:
-		return arm_priv(vcpu)->spsr_fiq;
-		break;
+		return p->spsr_fiq;
 	default:
 		break;
 	};
+
 	return 0;
 }
 
-int cpu_vcpu_spsr_update(struct vmm_vcpu *vcpu, 
+int cpu_vcpu_spsr_update(struct vmm_vcpu *vcpu,
 			 u32 new_spsr,
 			 u32 new_spsr_mask)
 {
+	struct arm_priv *p;
+
 	/* Sanity check */
 	if (!vcpu || !vcpu->is_normal) {
 		return VMM_EFAIL;
 	}
-	if ((arm_priv(vcpu)->cpsr & CPSR_MODE_MASK) == CPSR_MODE_USER) {
+	p = arm_priv(vcpu);
+
+	/* VCPU cannot be in user mode */
+	if ((p->cpsr & CPSR_MODE_MASK) == CPSR_MODE_USER) {
 		return VMM_EFAIL;
 	}
+
 	new_spsr &= new_spsr_mask;
+
 	/* Update appropriate SPSR */
-	switch (arm_priv(vcpu)->cpsr & CPSR_MODE_MASK) {
+	switch (p->cpsr & CPSR_MODE_MASK) {
 	case CPSR_MODE_ABORT:
-		arm_priv(vcpu)->spsr_abt &= ~new_spsr_mask;
-		arm_priv(vcpu)->spsr_abt |= new_spsr;
+		p->spsr_abt &= ~new_spsr_mask;
+		p->spsr_abt |= new_spsr;
 		break;
 	case CPSR_MODE_UNDEFINED:
-		arm_priv(vcpu)->spsr_und &= ~new_spsr_mask;
-		arm_priv(vcpu)->spsr_und |= new_spsr;
+		p->spsr_und &= ~new_spsr_mask;
+		p->spsr_und |= new_spsr;
 		break;
 	case CPSR_MODE_MONITOR:
-		arm_priv(vcpu)->spsr_mon &= ~new_spsr_mask;
-		arm_priv(vcpu)->spsr_mon |= new_spsr;
+		p->spsr_mon &= ~new_spsr_mask;
+		p->spsr_mon |= new_spsr;
 		break;
 	case CPSR_MODE_SUPERVISOR:
-		arm_priv(vcpu)->spsr_svc &= ~new_spsr_mask;
-		arm_priv(vcpu)->spsr_svc |= new_spsr;
+		p->spsr_svc &= ~new_spsr_mask;
+		p->spsr_svc |= new_spsr;
 		break;
 	case CPSR_MODE_IRQ:
-		arm_priv(vcpu)->spsr_irq &= ~new_spsr_mask;
-		arm_priv(vcpu)->spsr_irq |= new_spsr;
+		p->spsr_irq &= ~new_spsr_mask;
+		p->spsr_irq |= new_spsr;
 		break;
 	case CPSR_MODE_FIQ:
-		arm_priv(vcpu)->spsr_fiq &= ~new_spsr_mask;
-		arm_priv(vcpu)->spsr_fiq |= new_spsr;
+		p->spsr_fiq &= ~new_spsr_mask;
+		p->spsr_fiq |= new_spsr;
 		break;
 	default:
 		break;
 	};
+
 	/* Return success */
 	return VMM_OK;
 }
 
-u32 cpu_vcpu_reg_read(struct vmm_vcpu *vcpu, 
-		      arch_regs_t *regs, 
-		      u32 reg_num) 
+u32 cpu_vcpu_reg_read(struct vmm_vcpu *vcpu,
+		      arch_regs_t *regs,
+		      u32 reg_num)
 {
 	switch (reg_num) {
 	case 0:
@@ -372,12 +379,14 @@ u32 cpu_vcpu_reg_read(struct vmm_vcpu *vcpu,
 	return 0x0;
 }
 
-void cpu_vcpu_reg_write(struct vmm_vcpu *vcpu, 
-			arch_regs_t *regs, 
-			u32 reg_num, 
-			u32 reg_val) 
+void cpu_vcpu_reg_write(struct vmm_vcpu *vcpu,
+			arch_regs_t *regs,
+			u32 reg_num,
+			u32 reg_val)
 {
-	u32 curmode = arm_priv(vcpu)->cpsr & CPSR_MODE_MASK;
+	struct arm_priv *p = arm_priv(vcpu);
+	u32 curmode = p->cpsr & CPSR_MODE_MASK;
+
 	switch (reg_num) {
 	case 0:
 	case 1:
@@ -396,9 +405,9 @@ void cpu_vcpu_reg_write(struct vmm_vcpu *vcpu,
 	case 12:
 		regs->gpr[reg_num] = reg_val;
 		if (curmode == CPSR_MODE_FIQ) {
-			arm_priv(vcpu)->gpr_fiq[reg_num - 8] = reg_val;
+			p->gpr_fiq[reg_num - 8] = reg_val;
 		} else {
-			arm_priv(vcpu)->gpr_usr[reg_num - 8] = reg_val;
+			p->gpr_usr[reg_num - 8] = reg_val;
 		}
 		break;
 	case 13:
@@ -406,25 +415,25 @@ void cpu_vcpu_reg_write(struct vmm_vcpu *vcpu,
 		switch (curmode) {
 		case CPSR_MODE_USER:
 		case CPSR_MODE_SYSTEM:
-			arm_priv(vcpu)->sp_usr = reg_val;
+			p->sp_usr = reg_val;
 			break;
 		case CPSR_MODE_FIQ:
-			arm_priv(vcpu)->sp_fiq = reg_val;
+			p->sp_fiq = reg_val;
 			break;
 		case CPSR_MODE_IRQ:
-			arm_priv(vcpu)->sp_irq = reg_val;
+			p->sp_irq = reg_val;
 			break;
 		case CPSR_MODE_SUPERVISOR:
-			arm_priv(vcpu)->sp_svc = reg_val;
+			p->sp_svc = reg_val;
 			break;
 		case CPSR_MODE_ABORT:
-			arm_priv(vcpu)->sp_abt = reg_val;
+			p->sp_abt = reg_val;
 			break;
 		case CPSR_MODE_UNDEFINED:
-			arm_priv(vcpu)->sp_und = reg_val;
+			p->sp_und = reg_val;
 			break;
 		case CPSR_MODE_MONITOR:
-			arm_priv(vcpu)->sp_mon = reg_val;
+			p->sp_mon = reg_val;
 			break;
 		default:
 			break;
@@ -435,25 +444,25 @@ void cpu_vcpu_reg_write(struct vmm_vcpu *vcpu,
 		switch (curmode) {
 		case CPSR_MODE_USER:
 		case CPSR_MODE_SYSTEM:
-			arm_priv(vcpu)->lr_usr = reg_val;
+			p->lr_usr = reg_val;
 			break;
 		case CPSR_MODE_FIQ:
-			arm_priv(vcpu)->lr_fiq = reg_val;
+			p->lr_fiq = reg_val;
 			break;
 		case CPSR_MODE_IRQ:
-			arm_priv(vcpu)->lr_irq = reg_val;
+			p->lr_irq = reg_val;
 			break;
 		case CPSR_MODE_SUPERVISOR:
-			arm_priv(vcpu)->lr_svc = reg_val;
+			p->lr_svc = reg_val;
 			break;
 		case CPSR_MODE_ABORT:
-			arm_priv(vcpu)->lr_abt = reg_val;
+			p->lr_abt = reg_val;
 			break;
 		case CPSR_MODE_UNDEFINED:
-			arm_priv(vcpu)->lr_und = reg_val;
+			p->lr_und = reg_val;
 			break;
 		case CPSR_MODE_MONITOR:
-			arm_priv(vcpu)->lr_mon = reg_val;
+			p->lr_mon = reg_val;
 			break;
 		default:
 			break;
@@ -467,12 +476,14 @@ void cpu_vcpu_reg_write(struct vmm_vcpu *vcpu,
 	};
 }
 
-u32 cpu_vcpu_regmode_read(struct vmm_vcpu *vcpu, 
-			  arch_regs_t *regs, 
+u32 cpu_vcpu_regmode_read(struct vmm_vcpu *vcpu,
+			  arch_regs_t *regs,
 			  u32 mode,
 			  u32 reg_num)
 {
-	u32 curmode = arm_priv(vcpu)->cpsr & CPSR_MODE_MASK;
+	struct arm_priv *p = arm_priv(vcpu);
+	u32 curmode = p->cpsr & CPSR_MODE_MASK;
+
 	if (mode == curmode) {
 		return cpu_vcpu_reg_read(vcpu, regs, reg_num);
 	} else {
@@ -493,11 +504,10 @@ u32 cpu_vcpu_regmode_read(struct vmm_vcpu *vcpu,
 		case 11:
 		case 12:
 			if (curmode == CPSR_MODE_FIQ) {
-				return arm_priv(vcpu)->gpr_usr[reg_num - 8];
+				return p->gpr_usr[reg_num - 8];
 			} else {
 				if (mode == CPSR_MODE_FIQ) {
-					return arm_priv(vcpu)->
-							gpr_fiq[reg_num - 8];
+					return p->gpr_fiq[reg_num - 8];
 				} else {
 					return regs->gpr[reg_num];
 				}
@@ -507,25 +517,25 @@ u32 cpu_vcpu_regmode_read(struct vmm_vcpu *vcpu,
 			switch (mode) {
 			case CPSR_MODE_USER:
 			case CPSR_MODE_SYSTEM:
-				return arm_priv(vcpu)->sp_usr;
+				return p->sp_usr;
 				break;
 			case CPSR_MODE_FIQ:
-				return arm_priv(vcpu)->sp_fiq;
+				return p->sp_fiq;
 				break;
 			case CPSR_MODE_IRQ:
-				return arm_priv(vcpu)->sp_irq;
+				return p->sp_irq;
 				break;
 			case CPSR_MODE_SUPERVISOR:
-				return arm_priv(vcpu)->sp_svc;
+				return p->sp_svc;
 				break;
 			case CPSR_MODE_ABORT:
-				return arm_priv(vcpu)->sp_abt;
+				return p->sp_abt;
 				break;
 			case CPSR_MODE_UNDEFINED:
-				return arm_priv(vcpu)->sp_und;
+				return p->sp_und;
 				break;
 			case CPSR_MODE_MONITOR:
-				return arm_priv(vcpu)->sp_mon;
+				return p->sp_mon;
 				break;
 			default:
 				break;
@@ -535,25 +545,25 @@ u32 cpu_vcpu_regmode_read(struct vmm_vcpu *vcpu,
 			switch (mode) {
 			case CPSR_MODE_USER:
 			case CPSR_MODE_SYSTEM:
-				return arm_priv(vcpu)->lr_usr;
+				return p->lr_usr;
 				break;
 			case CPSR_MODE_FIQ:
-				return arm_priv(vcpu)->lr_fiq;
+				return p->lr_fiq;
 				break;
 			case CPSR_MODE_IRQ:
-				return arm_priv(vcpu)->lr_irq;
+				return p->lr_irq;
 				break;
 			case CPSR_MODE_SUPERVISOR:
-				return arm_priv(vcpu)->lr_svc;
+				return p->lr_svc;
 				break;
 			case CPSR_MODE_ABORT:
-				return arm_priv(vcpu)->lr_abt;
+				return p->lr_abt;
 				break;
 			case CPSR_MODE_UNDEFINED:
-				return arm_priv(vcpu)->lr_und;
+				return p->lr_und;
 				break;
 			case CPSR_MODE_MONITOR:
-				return arm_priv(vcpu)->lr_mon;
+				return p->lr_mon;
 				break;
 			default:
 				break;
@@ -566,16 +576,19 @@ u32 cpu_vcpu_regmode_read(struct vmm_vcpu *vcpu,
 			break;
 		};
 	}
+
 	return 0x0;
 }
 
-void cpu_vcpu_regmode_write(struct vmm_vcpu *vcpu, 
-			    arch_regs_t *regs, 
+void cpu_vcpu_regmode_write(struct vmm_vcpu *vcpu,
+			    arch_regs_t *regs,
 			    u32 mode,
 			    u32 reg_num,
 			    u32 reg_val)
 {
-	u32 curmode = arm_priv(vcpu)->cpsr & CPSR_MODE_MASK;
+	struct arm_priv *p = arm_priv(vcpu);
+	u32 curmode = p->cpsr & CPSR_MODE_MASK;
+
 	if (mode == curmode) {
 		cpu_vcpu_reg_write(vcpu, regs, reg_num, reg_val);
 	} else {
@@ -596,11 +609,10 @@ void cpu_vcpu_regmode_write(struct vmm_vcpu *vcpu,
 		case 11:
 		case 12:
 			if (curmode == CPSR_MODE_FIQ) {
-				arm_priv(vcpu)->gpr_usr[reg_num - 8] = reg_val;
+				p->gpr_usr[reg_num - 8] = reg_val;
 			} else {
 				if (mode == CPSR_MODE_FIQ) {
-					arm_priv(vcpu)->gpr_fiq[reg_num - 8] = 
-								reg_val;
+					p->gpr_fiq[reg_num - 8] = reg_val;
 				} else {
 					regs->gpr[reg_num] = reg_val;
 				}
@@ -610,25 +622,25 @@ void cpu_vcpu_regmode_write(struct vmm_vcpu *vcpu,
 			switch (mode) {
 			case CPSR_MODE_USER:
 			case CPSR_MODE_SYSTEM:
-				arm_priv(vcpu)->sp_usr = reg_val;
+				p->sp_usr = reg_val;
 				break;
 			case CPSR_MODE_FIQ:
-				arm_priv(vcpu)->sp_fiq = reg_val;
+				p->sp_fiq = reg_val;
 				break;
 			case CPSR_MODE_IRQ:
-				arm_priv(vcpu)->sp_irq = reg_val;
+				p->sp_irq = reg_val;
 				break;
 			case CPSR_MODE_SUPERVISOR:
-				arm_priv(vcpu)->sp_svc = reg_val;
+				p->sp_svc = reg_val;
 				break;
 			case CPSR_MODE_ABORT:
-				arm_priv(vcpu)->sp_abt = reg_val;
+				p->sp_abt = reg_val;
 				break;
 			case CPSR_MODE_UNDEFINED:
-				arm_priv(vcpu)->sp_und = reg_val;
+				p->sp_und = reg_val;
 				break;
 			case CPSR_MODE_MONITOR:
-				arm_priv(vcpu)->sp_mon = reg_val;
+				p->sp_mon = reg_val;
 				break;
 			default:
 				break;
@@ -638,25 +650,25 @@ void cpu_vcpu_regmode_write(struct vmm_vcpu *vcpu,
 			switch (mode) {
 			case CPSR_MODE_USER:
 			case CPSR_MODE_SYSTEM:
-				arm_priv(vcpu)->lr_usr = reg_val;
+				p->lr_usr = reg_val;
 				break;
 			case CPSR_MODE_FIQ:
-				arm_priv(vcpu)->lr_fiq = reg_val;
+				p->lr_fiq = reg_val;
 				break;
 			case CPSR_MODE_IRQ:
-				arm_priv(vcpu)->lr_irq = reg_val;
+				p->lr_irq = reg_val;
 				break;
 			case CPSR_MODE_SUPERVISOR:
-				arm_priv(vcpu)->lr_svc = reg_val;
+				p->lr_svc = reg_val;
 				break;
 			case CPSR_MODE_ABORT:
-				arm_priv(vcpu)->lr_abt = reg_val;
+				p->lr_abt = reg_val;
 				break;
 			case CPSR_MODE_UNDEFINED:
-				arm_priv(vcpu)->lr_und = reg_val;
+				p->lr_und = reg_val;
 				break;
 			case CPSR_MODE_MONITOR:
-				arm_priv(vcpu)->lr_mon = reg_val;
+				p->lr_mon = reg_val;
 				break;
 			default:
 				break;
@@ -679,9 +691,10 @@ int arch_guest_init(struct vmm_guest *guest)
 	struct cpu_page pg;
 
 	if (!guest->reset_count) {
-		guest->arch_priv = vmm_malloc(sizeof(arm_guest_priv_t));
+		guest->arch_priv = vmm_zalloc(sizeof(struct arm_guest_priv));
 		if (!guest->arch_priv) {
-			return VMM_EFAIL;
+			rc = VMM_EFAIL;
+			goto fail;
 		}
 		ovect_flags = 0x0;
 		ovect_flags |= VMM_MEMORY_READABLE;
@@ -690,13 +703,14 @@ int arch_guest_init(struct vmm_guest *guest)
 		ovect_flags |= VMM_MEMORY_EXECUTABLE;
 		ovect_va = vmm_host_alloc_pages(1, ovect_flags);
 		if (!ovect_va) {
-			return VMM_EFAIL;
+			rc = VMM_EFAIL;
+			goto fail;
 		}
 		if ((rc = cpu_mmu_get_reserved_page(ovect_va, &pg))) {
-			return rc;
+			goto fail_freepages;
 		}
 		if ((rc = cpu_mmu_unmap_reserved_page(&pg))) {
-			return rc;
+			goto fail_freepages;
 		}
 #if defined(CONFIG_ARMV5)
 		pg.ap = TTBL_AP_SRW_UR;
@@ -708,12 +722,27 @@ int arch_guest_init(struct vmm_guest *guest)
 		}
 #endif
 		if ((rc = cpu_mmu_map_reserved_page(&pg))) {
-			return rc;
+			goto fail_freepages;
 		}
 		arm_guest_priv(guest)->ovect = (u32 *)ovect_va;
+
+		if (vmm_devtree_read_u32(guest->node,
+				"psci_version",
+				 &arm_guest_priv(guest)->psci_version)) {
+			/* By default, assume PSCI v0.1 */
+			arm_guest_priv(guest)->psci_version = 1;
+		}
 	}
 
 	return VMM_OK;
+
+fail_freepages:
+	if (arm_guest_priv(guest)->ovect) {
+		vmm_host_free_pages(
+			(virtual_addr_t)arm_guest_priv(guest)->ovect, 1);
+	}
+fail:
+	return rc;
 }
 
 int arch_guest_deinit(struct vmm_guest *guest)
@@ -732,13 +761,25 @@ int arch_guest_deinit(struct vmm_guest *guest)
 	return VMM_OK;
 }
 
+int arch_guest_add_region(struct vmm_guest *guest, struct vmm_region *region)
+{
+	return VMM_OK;
+}
+
+int arch_guest_del_region(struct vmm_guest *guest, struct vmm_region *region)
+{
+	return VMM_OK;
+}
+
 int arch_vcpu_init(struct vmm_vcpu *vcpu)
 {
+	int rc;
 	u32 ite, cpuid;
 	const char *attr;
 
 	/* Initialize User Mode Registers */
 	/* For both Orphan & Normal VCPUs */
+
 	memset(arm_regs(vcpu), 0, sizeof(arch_regs_t));
 	arm_regs(vcpu)->pc = vcpu->start_pc;
 	arm_regs(vcpu)->sp_excp = vcpu->stack_va + vcpu->stack_sz - 4;
@@ -753,15 +794,17 @@ int arch_vcpu_init(struct vmm_vcpu *vcpu)
 		arm_regs(vcpu)->cpsr |= CPSR_MODE_SUPERVISOR;
 		arm_regs(vcpu)->sp = arm_regs(vcpu)->sp_excp;
 	}
+
 	/* Initialize Supervisor Mode Registers */
 	/* For only Normal VCPUs */
+
 	if (!vcpu->is_normal) {
 		return VMM_OK;
 	}
-	attr = vmm_devtree_attrval(vcpu->node, 
-				   VMM_DEVTREE_COMPATIBLE_ATTR_NAME);
-	if (!attr) {
-		return VMM_EFAIL;
+	rc = vmm_devtree_read_string(vcpu->node,
+			VMM_DEVTREE_COMPATIBLE_ATTR_NAME, &attr);
+	if (rc) {
+		goto fail;
 	}
 	if (strcmp(attr, "armv5te,arm926ej") == 0) {
 		cpuid = ARM_CPUID_ARM926;
@@ -772,13 +815,14 @@ int arch_vcpu_init(struct vmm_vcpu *vcpu)
 	} else if (strcmp(attr, "armv7a,cortex-a9") == 0) {
 		cpuid = ARM_CPUID_CORTEXA9;
 	} else {
-		return VMM_EFAIL;
+		rc = VMM_EINVALID;
+		goto fail;
 	}
 	if (!vcpu->reset_count) {
-		vcpu->arch_priv = vmm_zalloc(sizeof(arm_priv_t));
-		arm_priv(vcpu)->cpsr = CPSR_ASYNC_ABORT_DISABLED | 
+		vcpu->arch_priv = vmm_zalloc(sizeof(struct arm_priv));
+		arm_priv(vcpu)->cpsr = CPSR_ASYNC_ABORT_DISABLED |
 				   CPSR_IRQ_DISABLED |
-				   CPSR_FIQ_DISABLED | 
+				   CPSR_FIQ_DISABLED |
 				   CPSR_MODE_SUPERVISOR;
 	} else {
 		for (ite = 0; ite < CPU_FIQ_GPR_COUNT; ite++) {
@@ -805,12 +849,12 @@ int arch_vcpu_init(struct vmm_vcpu *vcpu)
 		arm_priv(vcpu)->sp_fiq = 0x0;
 		arm_priv(vcpu)->lr_fiq = 0x0;
 		arm_priv(vcpu)->spsr_fiq = 0x0;
-		cpu_vcpu_cpsr_update(vcpu, 
-				     arm_regs(vcpu), 
+		cpu_vcpu_cpsr_update(vcpu,
+				     arm_regs(vcpu),
 				     (CPSR_ZERO_MASK |
-					CPSR_ASYNC_ABORT_DISABLED | 
+					CPSR_ASYNC_ABORT_DISABLED |
 					CPSR_IRQ_DISABLED |
-					CPSR_FIQ_DISABLED | 
+					CPSR_FIQ_DISABLED |
 					CPSR_MODE_SUPERVISOR),
 				     CPSR_ALLBITS_MASK);
 	}
@@ -828,6 +872,7 @@ int arch_vcpu_init(struct vmm_vcpu *vcpu)
 			arm_set_feature(vcpu, ARM_FEATURE_V6K);
 			arm_set_feature(vcpu, ARM_FEATURE_VFP);
 			arm_set_feature(vcpu, ARM_FEATURE_VAPA);
+			arm_set_feature(vcpu, ARM_FEATURE_MPIDR);
 			arm_set_feature(vcpu, ARM_FEATURE_DUMMY_C15_REGS);
 			break;
 		case ARM_CPUID_CORTEXA8:
@@ -850,7 +895,6 @@ int arch_vcpu_init(struct vmm_vcpu *vcpu)
 		default:
 			break;
 		};
-
 		/* Some features automatically imply others: */
 		if (arm_feature(vcpu, ARM_FEATURE_V7)) {
 			arm_set_feature(vcpu, ARM_FEATURE_VAPA);
@@ -892,7 +936,38 @@ int arch_vcpu_init(struct vmm_vcpu *vcpu)
 		}
 	}
 
-	return cpu_vcpu_cp15_init(vcpu, cpuid);
+	rc = cpu_vcpu_vfp_init(vcpu);
+	if (rc) {
+		goto fail_vfp_init;
+	}
+
+	rc = cpu_vcpu_cp14_init(vcpu);
+	if (rc) {
+		goto fail_cp14_init;
+	}
+
+	rc = cpu_vcpu_cp15_init(vcpu, cpuid);
+	if (rc) {
+		goto fail_cp15_init;
+	}
+
+	return VMM_OK;
+
+fail_cp15_init:
+	if (!vcpu->reset_count) {
+		cpu_vcpu_cp14_deinit(vcpu);
+	}
+fail_cp14_init:
+	if (!vcpu->reset_count) {
+		cpu_vcpu_vfp_deinit(vcpu);
+	}
+fail_vfp_init:
+	if (!vcpu->reset_count) {
+		vmm_free(vcpu->arch_priv);
+		vcpu->arch_priv = NULL;
+	}
+fail:
+	return rc;
 }
 
 int arch_vcpu_deinit(struct vmm_vcpu *vcpu)
@@ -912,6 +987,16 @@ int arch_vcpu_deinit(struct vmm_vcpu *vcpu)
 		return rc;
 	}
 
+	/* Cleanup CP14 */
+	if ((rc = cpu_vcpu_cp14_deinit(vcpu))) {
+		return rc;
+	}
+
+	/* Cleanup VFP */
+	if ((rc = cpu_vcpu_vfp_deinit(vcpu))) {
+		return rc;
+	}
+
 	/* Free super regs */
 	vmm_free(vcpu->arch_priv);
 
@@ -919,7 +1004,7 @@ int arch_vcpu_deinit(struct vmm_vcpu *vcpu)
 }
 
 void arch_vcpu_switch(struct vmm_vcpu *tvcpu,
-		      struct vmm_vcpu *vcpu, 
+		      struct vmm_vcpu *vcpu,
                       arch_regs_t *regs)
 {
 	u32 ite;
@@ -933,12 +1018,16 @@ void arch_vcpu_switch(struct vmm_vcpu *tvcpu,
 		}
 		arm_regs(tvcpu)->cpsr = regs->cpsr;
 		arm_regs(tvcpu)->sp_excp = regs->sp_excp;
-		if(tvcpu->is_normal) {
-			cpu_vcpu_banked_regs_save(tvcpu, regs);
+		if (tvcpu->is_normal) {
+			cpu_vcpu_banked_regs_save(arm_priv(tvcpu), regs);
+			/* Save VFP regs */
+			cpu_vcpu_vfp_regs_save(tvcpu);
+			/* Save CP14 regs */
+			cpu_vcpu_cp14_regs_save(tvcpu);
+			/* Save CP15 regs */
+			cpu_vcpu_cp15_regs_save(tvcpu);
 		}
 	}
-	/* Switch CP15 context */
-	cpu_vcpu_cp15_switch_context(tvcpu, vcpu);
 	/* Restore user registers & banked registers */
 	regs->pc = arm_regs(vcpu)->pc;
 	regs->lr = arm_regs(vcpu)->lr;
@@ -949,12 +1038,34 @@ void arch_vcpu_switch(struct vmm_vcpu *tvcpu,
 	regs->cpsr = arm_regs(vcpu)->cpsr;
 	regs->sp_excp = arm_regs(vcpu)->sp_excp;
 	if (vcpu->is_normal) {
-		cpu_vcpu_banked_regs_restore(vcpu, regs);
+		/* Restore VFP regs */
+		cpu_vcpu_vfp_regs_restore(vcpu);
+		/* Restore CP14 regs */
+		cpu_vcpu_cp14_regs_restore(vcpu);
+		/* Restore CP15 regs */
+		cpu_vcpu_cp15_regs_restore(vcpu);
+		/* Restore banked registers */
+		cpu_vcpu_banked_regs_restore(arm_priv(vcpu), regs);
+	} else {
+		/* Restore hypervisor TTBL for Orphan VCPUs */
+		if (tvcpu) {
+			if (tvcpu->is_normal) {
+				cpu_mmu_change_ttbr(cpu_mmu_l1tbl_default());
+			}
+		} else {
+			cpu_mmu_change_ttbr(cpu_mmu_l1tbl_default());
+		}
 	}
 	/* Clear exclusive monitor */
 	clrex();
 }
 
+void arch_vcpu_post_switch(struct vmm_vcpu *vcpu,
+			   arch_regs_t *regs)
+{
+	/* Nothing to do here. */
+}
+
 void arch_vcpu_preempt_orphan(void)
 {
 	/* Trigger SVC call from supervisor mode. This will cause
@@ -963,20 +1074,25 @@ void arch_vcpu_preempt_orphan(void)
 	asm volatile ("svc #0\t\n");
 }
 
-static void __cpu_vcpu_dump_user_reg(struct vmm_chardev *cdev, 
+static void __cpu_vcpu_dump_user_reg(struct vmm_chardev *cdev,
 				     struct vmm_vcpu *vcpu, arch_regs_t *regs)
 {
-	u32 ite;
-	vmm_cprintf(cdev, "  Core Registers\n");
-	vmm_cprintf(cdev, "    SP=0x%08x       LR=0x%08x       PC=0x%08x\n",
-		    regs->sp, regs->lr, regs->pc);
-	vmm_cprintf(cdev, "    CPSR=0x%08x     \n", 
-		    cpu_vcpu_cpsr_retrieve(vcpu, regs));
-	vmm_cprintf(cdev, "  General Purpose Registers");
-	for (ite = 0; ite < CPU_GPR_COUNT; ite++) {
-		if (ite % 3 == 0)
+	u32 i;
+
+	vmm_cprintf(cdev, "Core Registers\n");
+	vmm_cprintf(cdev, " %7s=0x%08x %7s=0x%08x %7s=0x%08x\n",
+		    "SP", regs->sp,
+		    "LR", regs->lr,
+		    "PC", regs->pc);
+	vmm_cprintf(cdev, " %7s=0x%08x\n",
+		    "CPSR", cpu_vcpu_cpsr_retrieve(vcpu, regs));
+	vmm_cprintf(cdev, "General Purpose Registers");
+	for (i = 0; i < CPU_GPR_COUNT; i++) {
+		if (i % 3 == 0) {
 			vmm_cprintf(cdev, "\n");
-		vmm_cprintf(cdev, "    R%02d=0x%08x  ", ite, regs->gpr[ite]);
+		}
+		vmm_cprintf(cdev, " %5s%02d=0x%08x",
+			    "R", i, regs->gpr[i]);
 	}
 	vmm_cprintf(cdev, "\n");
 }
@@ -988,7 +1104,7 @@ void cpu_vcpu_dump_user_reg(struct vmm_vcpu *vcpu, arch_regs_t *regs)
 
 void arch_vcpu_regs_dump(struct vmm_chardev *cdev, struct vmm_vcpu *vcpu)
 {
-	u32 ite;
+	u32 i;
 
 	/* For both Normal & Orphan VCPUs */
 	__cpu_vcpu_dump_user_reg(cdev, vcpu, arm_regs(vcpu));
@@ -998,40 +1114,58 @@ void arch_vcpu_regs_dump(struct vmm_chardev *cdev, struct vmm_vcpu *vcpu)
 		return;
 	}
 
-	vmm_cprintf(cdev, "  User Mode Registers (Banked)\n");
-	vmm_cprintf(cdev, "    SP=0x%08x       LR=0x%08x\n",
-		    arm_priv(vcpu)->sp_usr, arm_priv(vcpu)->lr_usr);
-	vmm_cprintf(cdev, "  Supervisor Mode Registers (Banked)\n");
-	vmm_cprintf(cdev, "    SP=0x%08x       LR=0x%08x       SPSR=0x%08x\n",
-		    arm_priv(vcpu)->sp_svc, arm_priv(vcpu)->lr_svc,
-		    arm_priv(vcpu)->spsr_svc);
-	vmm_cprintf(cdev, "  Monitor Mode Registers (Banked)\n");
-	vmm_cprintf(cdev, "    SP=0x%08x       LR=0x%08x       SPSR=0x%08x\n",
-		    arm_priv(vcpu)->sp_mon, arm_priv(vcpu)->lr_mon,
-		    arm_priv(vcpu)->spsr_mon);
-	vmm_cprintf(cdev, "  Abort Mode Registers (Banked)\n");
-	vmm_cprintf(cdev, "    SP=0x%08x       LR=0x%08x       SPSR=0x%08x\n",
-		    arm_priv(vcpu)->sp_abt, arm_priv(vcpu)->lr_abt,
-		    arm_priv(vcpu)->spsr_abt);
-	vmm_cprintf(cdev, "  Undefined Mode Registers (Banked)\n");
-	vmm_cprintf(cdev, "    SP=0x%08x       LR=0x%08x       SPSR=0x%08x\n",
-		    arm_priv(vcpu)->sp_und, arm_priv(vcpu)->lr_und,
-		    arm_priv(vcpu)->spsr_und);
-	vmm_cprintf(cdev, "  IRQ Mode Registers (Banked)\n");
-	vmm_cprintf(cdev, "    SP=0x%08x       LR=0x%08x       SPSR=0x%08x\n",
-		    arm_priv(vcpu)->sp_irq, arm_priv(vcpu)->lr_irq,
-		    arm_priv(vcpu)->spsr_irq);
-	vmm_cprintf(cdev, "  FIQ Mode Registers (Banked)\n");
-	vmm_cprintf(cdev, "    SP=0x%08x       LR=0x%08x       SPSR=0x%08x",
-		    arm_priv(vcpu)->sp_fiq, arm_priv(vcpu)->lr_fiq,
-		    arm_priv(vcpu)->spsr_fiq);
-	for (ite = 0; ite < 5; ite++) {
-		if (ite % 3 == 0)
+	/* Print banked registers */
+	vmm_cprintf(cdev, "User Mode Registers (Banked)\n");
+	vmm_cprintf(cdev, " %7s=0x%08x %7s=0x%08x\n",
+		    "SP", arm_priv(vcpu)->sp_usr,
+		    "LR", arm_priv(vcpu)->lr_usr);
+	vmm_cprintf(cdev, "Supervisor Mode Registers (Banked)\n");
+	vmm_cprintf(cdev, " %7s=0x%08x %7s=0x%08x %7s=0x%08x\n",
+		    "SP", arm_priv(vcpu)->sp_svc,
+		    "LR", arm_priv(vcpu)->lr_svc,
+		    "SPSR", arm_priv(vcpu)->spsr_svc);
+	vmm_cprintf(cdev, "Monitor Mode Registers (Banked)\n");
+	vmm_cprintf(cdev, " %7s=0x%08x %7s=0x%08x %7s=0x%08x\n",
+		    "SP", arm_priv(vcpu)->sp_mon,
+		    "LR", arm_priv(vcpu)->lr_mon,
+		    "SPSR", arm_priv(vcpu)->spsr_mon);
+	vmm_cprintf(cdev, "Abort Mode Registers (Banked)\n");
+	vmm_cprintf(cdev, " %7s=0x%08x %7s=0x%08x %7s=0x%08x\n",
+		    "SP", arm_priv(vcpu)->sp_abt,
+		    "LR", arm_priv(vcpu)->lr_abt,
+		    "SPSR", arm_priv(vcpu)->spsr_abt);
+	vmm_cprintf(cdev, "Undefined Mode Registers (Banked)\n");
+	vmm_cprintf(cdev, " %7s=0x%08x %7s=0x%08x %7s=0x%08x\n",
+		    "SP", arm_priv(vcpu)->sp_und,
+		    "LR", arm_priv(vcpu)->lr_und,
+		    "SPSR", arm_priv(vcpu)->spsr_und);
+	vmm_cprintf(cdev, "IRQ Mode Registers (Banked)\n");
+	vmm_cprintf(cdev, " %7s=0x%08x %7s=0x%08x %7s=0x%08x\n",
+		    "SP", arm_priv(vcpu)->sp_irq,
+		    "LR", arm_priv(vcpu)->lr_irq,
+		    "SPSR", arm_priv(vcpu)->spsr_irq);
+	vmm_cprintf(cdev, "FIQ Mode Registers (Banked)\n");
+	vmm_cprintf(cdev, " %7s=0x%08x %7s=0x%08x %7s=0x%08x",
+		    "SP", arm_priv(vcpu)->sp_fiq,
+		    "LR", arm_priv(vcpu)->lr_fiq,
+		    "SPSR", arm_priv(vcpu)->spsr_fiq);
+	for (i = 0; i < 5; i++) {
+		if (i % 3 == 0) {
 			vmm_cprintf(cdev, "\n");
-		vmm_cprintf(cdev, "    R%02d=0x%08x  ", (ite + 8),
-			    arm_priv(vcpu)->gpr_fiq[ite]);
+		}
+		vmm_cprintf(cdev, " %5s%02d=0x%08x",
+			   "R", (i + 8), arm_priv(vcpu)->gpr_fiq[i]);
 	}
 	vmm_cprintf(cdev, "\n");
+
+	/* Print VFP registers */
+	cpu_vcpu_vfp_regs_dump(cdev, vcpu);
+
+	/* Print CP14 registers */
+	cpu_vcpu_cp14_regs_dump(cdev, vcpu);
+
+	/* Print CP15 registers */
+	cpu_vcpu_cp15_regs_dump(cdev, vcpu);
 }
 
 void arch_vcpu_stat_dump(struct vmm_chardev *cdev, struct vmm_vcpu *vcpu)
