@@ -964,7 +964,7 @@ void __handle_cpuid(struct vcpu_hw_context *context)
 	case CPUID_EXTENDED_BRANDSTRING:
 	case CPUID_EXTENDED_BRANDSTRINGMORE:
 	case CPUID_EXTENDED_BRANDSTRINGEND:
-	case CPUID_EXTENDED_L1_CACHE_TLB_IDENTIFIER:
+	case AMD_CPUID_EXTENDED_L1_CACHE_TLB_IDENTIFIER:
 	case CPUID_EXTENDED_L2_CACHE_TLB_IDENTIFIER:
 		func = &priv->extended_funcs[context->vmcb->rax
 					     - CPUID_EXTENDED_BASE];
@@ -976,10 +976,10 @@ void __handle_cpuid(struct vcpu_hw_context *context)
 
 	case CPUID_BASE_FEAT_FLAGS:
 	case CPUID_EXTENDED_FEATURES:
-	case CPUID_EXTENDED_ADDR_NR_PROC:
+	case AMD_CPUID_EXTENDED_ADDR_NR_PROC:
 	case CPUID_EXTENDED_CAPABILITIES:
 	case CPUID_BASE_PWR_MNG:
-	case CPUID_EXTENDED_SVM_IDENTIFIER:
+	case AMD_CPUID_EXTENDED_SVM_IDENTIFIER:
 		context->vmcb->rax = 0;
 		context->g_regs[GUEST_REGS_RBX] = 0;
 		context->g_regs[GUEST_REGS_RCX] = 0;
