@@ -25,37 +25,33 @@
 #ifndef __VMM_TYPES_H__
 #define __VMM_TYPES_H__
 
-#include <arch_types.h>
-
 typedef char s8;
 typedef short s16;
 typedef int s32;
 typedef unsigned char u8;
 typedef unsigned short u16;
 typedef unsigned int u32;
-typedef unsigned int size_t;
-typedef unsigned int bool;
-typedef unsigned int ulong;
 typedef long long s64;
 typedef unsigned long long u64;
 
+typedef unsigned int bool;
+typedef unsigned int size_t;
+typedef signed int ssize_t;
+typedef unsigned long ulong;
+typedef signed int off_t;
+typedef signed long long loff_t;
+
 /** Boolean macros */
 #define TRUE			1
 #define FALSE			0
+#define true			TRUE
+#define false			FALSE	
 #define NULL 			((void *)0)
 
-#define stringify(s)		tostring(s)
-#define tostring(s)		#s
-#define barrier() 		__asm__ __volatile__("": : :"memory")
-
-#define __alwaysinline 		__attribute__((always_inline))
-#define __unused		__attribute__((unused))
-#define __noreturn		__attribute__((noreturn))
-#define __notrace		__attribute__((no_instrument_function))
-
-#define __lock			__attribute__((section(".spinlock.text")))
-#define __modtbl		__attribute__((section(".modtbl")))
-#define __init			__attribute__((section(".init.text")))
+#include <arch_types.h>
 
+typedef physical_addr_t dma_addr_t;
+typedef physical_addr_t resource_addr_t;
+typedef physical_size_t resource_size_t;
 
 #endif /* __VMM_TYPES_H__ */
