@@ -22,10 +22,12 @@
  */
 
 #include <arm_io.h>
+#include <arm_cache.h>
 #include <arm_heap.h>
 #include <arm_mmu.h>
 #include <arm_irq.h>
 #include <arm_timer.h>
+#include <arm_math.h>
 #include <arm_string.h>
 #include <arm_stdio.h>
 #include <arm_board.h>
@@ -92,26 +94,26 @@ void arm_cmd_help(int argc, char **argv)
 	arm_puts("            <count> = byte count in hex\n");
 	arm_puts("\n");
 	arm_puts("start_linux - Start linux kernel (atags mechanism)\n");
-	arm_puts("            Usage: start_linux <kernel_addr> <initrd_addr> <initrd_size>\n");
+	arm_puts("            Usage: start_linux <kernel_addr> [<initrd_addr>] [<initrd_size>]\n");
 	arm_puts("            <kernel_addr>  = kernel load address\n");
-	arm_puts("            <initrd_addr>  = initrd load address\n");
-	arm_puts("            <initrd_size>  = initrd size\n");
+	arm_puts("            <initrd_addr>  = initrd load address (optional)\n");
+	arm_puts("            <initrd_size>  = initrd size (optional)\n");
 	arm_puts("\n");
 #ifdef BOARD_FDT_SUPPORT
 	arm_puts("start_linux_fdt - Start linux kernel (device-tree mechanism)\n");
-	arm_puts("            Usage: start_linux_fdt <kernel_addr> <initrd_addr> <initrd_size> <fdt_addr>\n");
+	arm_puts("            Usage: start_linux_fdt <kernel_addr> <fdt_addr> [<initrd_addr>] [<initrd_size>]\n");
 	arm_puts("            <kernel_addr>  = kernel load address\n");
-	arm_puts("            <initrd_addr>  = initrd load address\n");
-	arm_puts("            <initrd_size>  = initrd size\n");
-	arm_puts("            <fdt_size>     = fdt blob size\n");
+	arm_puts("            <fdt_addr>     = fdt blob address\n");
+	arm_puts("            <initrd_addr>  = initrd load address (optional)\n");
+	arm_puts("            <initrd_size>  = initrd size (optional)\n");
 	arm_puts("\n");
 #endif
 	arm_puts("linux_cmdline - Show/Update linux command line\n");
-	arm_puts("            Usage: linux_cmdline <new_linux_cmdline> \n");
+	arm_puts("            Usage: linux_cmdline [<new_linux_cmdline>]\n");
 	arm_puts("            <new_linux_cmdline>  = linux command line\n");
 	arm_puts("\n");
 	arm_puts("linux_memory_size - Show/Update linux memory size\n");
-	arm_puts("            Usage: linux_memory_size <memory_size> \n");
+	arm_puts("            Usage: linux_memory_size [<memory_size>]\n");
 	arm_puts("            <memory_size>  = memory size in hex\n");
 	arm_puts("\n");
 	arm_puts("autoexec    - autoexec command list from flash\n");
@@ -273,10 +275,10 @@ void arm_cmd_timer(int argc, char **argv)
 	arm_ulonglong2hexstr(str, irq_count);
 	arm_puts(str);
 	arm_puts("\n");
-	arm_puts("  IRQ Delay:  0x");
-	arm_ulonglong2hexstr(str, irq_delay);
+	arm_puts("  IRQ Delay:  ");
+	arm_ulonglong2str(str, irq_delay);
 	arm_puts(str);
-	arm_puts("\n");
+	arm_puts(" nsecs\n");
 	arm_puts("  Time Stamp: 0x");
 	arm_ulonglong2hexstr(str, tstamp);
 	arm_puts(str);
@@ -346,50 +348,108 @@ void arm_cmd_copy(int argc, char **argv)
 {
 	u64 tstamp;
 	char time[256];
-	u8 *dest, *src;
+	u32 *dest, *src;
+	virtual_addr_t dest_va;
 	u32 i, count;
+
+	/* Determine copy args */
 	if (argc != 4) {
 		arm_puts ("copy: must provide <dest>, <src>, and <count>\n");
 		return;
 	}
-	dest = (u8 *)arm_hexstr2uint(argv[1]);
-	src = (u8 *)arm_hexstr2uint(argv[2]);
+	dest = (u32 *)arm_hexstr2uint(argv[1]);
+	if (((virtual_addr_t)dest) & 0x3) {
+		arm_puts ("copy: <dest> should be 4-byte aligned\n");
+		return;
+	}
+	dest_va = (virtual_addr_t)dest;
+	src = (u32 *)arm_hexstr2uint(argv[2]);
+	if (((virtual_addr_t)src) & 0x3) {
+		arm_puts ("copy: <src> should be 4-byte aligned\n");
+		return;
+	}
 	count = arm_hexstr2uint(argv[3]);
+
+	/* Disable timer and get start timestamp */
 	arm_timer_disable();
 	tstamp = arm_timer_timestamp();
-	for (i = 0; i < count; i++) {
+
+	/* It might happen that we are running Basic firmware
+	 * after a reboot from Guest Linux in which case both
+	 * I-Cache and D-Cache will have stale contents. We need
+	 * to cleanup these stale contents while copying so that
+	 * we see correct contents of destination even after
+	 * MMU ON.
+	 */
+	arm_clean_invalidate_dcache_mva_range(dest_va, dest_va + count);
+
+	/* Copy contents */
+	for (i = 0; i < (count/sizeof(*dest)); i++) {
 		dest[i] = src[i];
 	}
+
+	/* Enable timer and get end timestamp */
 	tstamp = arm_timer_timestamp() - tstamp;
+	tstamp = arm_udiv64(tstamp, 1000);
 	arm_timer_enable();
+
+	/* Print time taken */
 	arm_ulonglong2str(time, tstamp);
 	arm_puts("copy took ");
 	arm_puts(time);
-	arm_puts(" ns for ");
+	arm_puts(" usecs for ");
 	arm_puts(argv[3]);
 	arm_puts(" bytes\n");
 }
 
 static char  cmdline[1024];
 
-typedef void (* linux_entry_t) (u32 zero, u32 machine_type, u32 kernel_args, u32 zero2);
+typedef void (*linux_entry_t) (u32 zero, u32 machine_type, u32 kernel_args, u32 zero2);
 
 void arm_cmd_start_linux(int argc, char **argv)
 {
 	char cfg_str[32];
-	u32 * kernel_args = (u32 *)(arm_board_ram_start() + 0x1000);
+	u32 *kernel_args = (u32 *)(arm_board_ram_start() + 0x1000);
 	u32 cmdline_size, p;
 	u32 kernel_addr, initrd_addr, initrd_size;
+	virtual_addr_t nuke_va;
 
-	if (argc != 4) {
-		arm_puts ("start_linux: must provide <kernel_addr>, <initrd_addr>, and <initrd_size>\n");
+	if (argc < 2) {
+		arm_puts ("start_linux: must provide <kernel_addr>\n");
+		arm_puts ("start_linux: <initrd_addr> and <initrd_size> are optional\n");
 		return;
 	}
 
 	/* Parse the arguments from command line */
 	kernel_addr = arm_hexstr2uint(argv[1]);
-	initrd_addr = arm_hexstr2uint(argv[2]);
-	initrd_size = arm_hexstr2uint(argv[3]);
+	if (argc > 2) {
+		initrd_addr = arm_hexstr2uint(argv[2]);
+	} else {
+		initrd_addr = 0;
+	}
+	if (argc > 3) {
+		initrd_size = arm_hexstr2uint(argv[3]);
+	} else {
+		initrd_size = 0;
+	}
+
+	/* Linux ARM32 kernel expects us to boot from 0x8000
+	 * aligned address, perferrably RAM start + 0x8000 address.
+	 * The 0x8000 bytes above kernel start address is used by
+	 * Linux ARM32 kernel to setup boot page tables.
+	 *
+	 * It might happen that we are running Basic firmware
+	 * after a reboot from Guest Linux in which case both
+	 * I-Cache and D-Cache will have stale contents. If we
+	 * don't cleanup these stale contents then Linux kernel
+	 * will not see correct contents boot page tables after
+	 * MMU ON.
+	 *
+	 * To take care of above described issue, we nuke the
+	 * 1MB area containing kernel start and boot page tables.
+	 */
+	nuke_va = kernel_addr & ~(0x100000 - 1);
+	arm_clean_invalidate_dcache_mva_range(nuke_va, nuke_va + 0x100000);
 
 	/* Setup kernel args */
 	for (p = 0; p < 128; p++) {
@@ -408,10 +468,12 @@ void arm_cmd_start_linux(int argc, char **argv)
 	kernel_args[p++] = memory_size;
 	kernel_args[p++] = arm_board_ram_start();
 	/* ATAG_INITRD2 */
-	kernel_args[p++] = 4;
-	kernel_args[p++] = 0x54420005;
-	kernel_args[p++] = initrd_addr;
-	kernel_args[p++] = initrd_size;
+	if (initrd_size) {
+		kernel_args[p++] = 4;
+		kernel_args[p++] = 0x54420005;
+		kernel_args[p++] = initrd_addr;
+		kernel_args[p++] = initrd_size;
+	}
 
 	/* Pass memory size via kernel command line */
 	arm_sprintf(cfg_str, " mem=%dM", (memory_size >> 20));
@@ -454,17 +516,45 @@ void arm_cmd_start_linux_fdt(int argc, char **argv)
 	u32 kernel_addr, fdt_addr;
 	u32 initrd_addr, initrd_size;
 	char cfg_str[32];
+	virtual_addr_t nuke_va;
 
-	if (argc != 5) {
-		arm_puts ("start_linux: must provide <kernel_addr>, <initrd_addr>, <initrd_size> and <fdt_addr>\n");
+	if (argc < 3) {
+		arm_puts ("start_linux_fdt: must provide <kernel_addr> and <fdt_addr>\n");
+		arm_puts ("start_linux_fdt: <initrd_addr> and <initrd_size> are optional\n");
 		return;
 	}
 
 	/* Parse the arguments from command line */
 	kernel_addr = arm_hexstr2uint(argv[1]);
-	fdt_addr = arm_hexstr2uint(argv[4]);
-	initrd_addr = arm_hexstr2uint(argv[2]);
-	initrd_size = arm_hexstr2uint(argv[3]);
+	fdt_addr = arm_hexstr2uint(argv[2]);
+	if (argc > 3) {
+		initrd_addr = arm_hexstr2uint(argv[3]);
+	} else {
+		initrd_addr = 0;
+	}
+	if (argc > 4) {
+		initrd_size = arm_hexstr2uint(argv[4]);
+	} else {
+		initrd_size = 0;
+	}
+
+	/* Linux ARM32 kernel expects us to boot from 0x8000
+	 * aligned address, perferrably RAM start + 0x8000 address.
+	 * The 0x8000 bytes above kernel start address is used by
+	 * Linux ARM32 kernel to setup boot page tables.
+	 *
+	 * It might happen that we are running Basic firmware
+	 * after a reboot from Guest Linux in which case both
+	 * I-Cache and D-Cache will have stale contents. If we
+	 * don't cleanup these stale contents then Linux kernel
+	 * will not see correct contents boot page tables after
+	 * MMU ON.
+	 *
+	 * To take care of above described issue, we nuke the
+	 * 1MB area containing kernel start and boot page tables.
+	 */
+	nuke_va = kernel_addr & ~(0x100000 - 1);
+	arm_clean_invalidate_dcache_mva_range(nuke_va, nuke_va + 0x100000);
 
 	/* Pass memory size via kernel command line */
 	arm_sprintf(cfg_str, " mem=%dM", (memory_size >> 20));
@@ -475,7 +565,10 @@ void arm_cmd_start_linux_fdt(int argc, char **argv)
 	 * 		- kernel cmd line
 	 * 		- number of cpus   */
 	fdt_chosen((void *)fdt_addr, 1, cmdline);
-	fdt_initrd((void *)fdt_addr, initrd_addr, initrd_addr + initrd_size, 1);
+	if (initrd_size) {
+		fdt_initrd((void *)fdt_addr,
+			   initrd_addr, initrd_addr + initrd_size, 1);
+	}
 
 	/* Disable interrupts and timer */
 	arm_timer_disable();
@@ -535,11 +628,12 @@ void arm_exec(char *line);
 
 void arm_cmd_autoexec(int argc, char **argv)
 {
+#define ARM_CMD_AUTOEXEC_BUF_SIZE	4096
 	static int lock = 0;
-	int len;
+	int len, pos = 0;
 	/* commands to execute are stored in NOR flash */
 	char *ptr = (char *)(arm_board_flash_addr() + 0xFF000);
-	char buffer[4096];
+	char buffer[ARM_CMD_AUTOEXEC_BUF_SIZE];
 
 	if (argc != 1) {
 		arm_puts ("autoexec: no parameters required\n");
@@ -554,33 +648,46 @@ void arm_cmd_autoexec(int argc, char **argv)
 
 	lock = 1;
 
-	if ((len = arm_strlen(ptr))) {
-		int pos = 0;
+	/* determine length of command list */
+	len = 0;
+	while ((len < ARM_CMD_AUTOEXEC_BUF_SIZE) &&
+	       arm_isprintable(ptr[len])) {
+		len++;
+	}
 
-		/* copy commands from NOR flash */
-		arm_strcpy(buffer, ptr);
+	/* sanity check on command list */
+	if (!len) {
+		arm_puts("command list not found !!!\n");
+		return;
+	}
+	if (len >= ARM_CMD_AUTOEXEC_BUF_SIZE) {
+		arm_printf("command list len=%d too big !!!\n", len);
+		return;
+	}
 
-		/* now we process them */
-		while (pos < len) {
-			ptr = &buffer[pos];
+	/* copy commands from NOR flash */
+	arm_memcpy(buffer, ptr, len);
+	buffer[len] = '\0';
 
-			/* We need to separate the commands */
-			while ((buffer[pos] != '\r') &&
-				(buffer[pos] != '\n') &&
-				(buffer[pos] != 0)) {
-				pos++;
-			}
-			buffer[pos] = '\0';
+	/* now we process them */
+	while (pos < len) {
+		ptr = &buffer[pos];
+
+		/* We need to separate the commands */
+		while ((buffer[pos] != '\r') &&
+			(buffer[pos] != '\n') &&
+			(buffer[pos] != 0)) {
 			pos++;
-
-			/* print the command */
-			arm_puts("autoexec(");
-			arm_puts(ptr);
-			arm_puts(")\n");
-			/* execute it */
-			arm_exec(ptr);
 		}
+		buffer[pos] = '\0';
+		pos++;
 
+		/* print the command */
+		arm_puts("autoexec(");
+		arm_puts(ptr);
+		arm_puts(")\n");
+		/* execute it */
+		arm_exec(ptr);
 	}
 
 	lock = 0;
